//+------------------------------------------------------------------+
//|                                               TimeGMT_TestEA.mq5 |
//|                                        Copyright © 2018, Amr Ali |
//|                             https://www.mql5.com/en/users/amrali |
//+------------------------------------------------------------------+
#property copyright "Copyright © 2018, Amr Ali"
#property link      "https://www.mql5.com/en/users/amrali"
#property version "1.35"

// Uncomment the following line to see debug messages
//#define _DEBUG

#include "TimeGMT.mqh"

input int timer_minutes = 1;   // Timer minutes for strategy tester

datetime previous_tc = 0;
datetime previous_tts = 0;
datetime previous_gmt = 0;
int      previous_brokeroffset = -1;

//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
int OnInit()
  {
   EventSetTimer(timer_minutes * 60);  // use timer to backtest weekends
   show();
   return (INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//| Expert Timer function                                            |
//+------------------------------------------------------------------+
void OnTimer()
  {
   show();
  }
//+------------------------------------------------------------------+
//| Time macros                                                      |
//+------------------------------------------------------------------+
#define  HOUR (60*60)
#define  DAY  (24*HOUR)
#define  WEEK (7*DAY)
string  _WkDys[] = {"Sun","Mon","Tue","Wed","Thu","Fri","Sat"};
#define  DoWi(t) ((int)((t)/DAY+4)%7)                       // Day of Week as integer:  Sun=0, Mon=1, Tue=2, ...
#define  DoWs(t) (_WkDy[DoWi(t)])                           // Day of Week as string:  Sun, Mon, Tue, ...
#define  t2s(t) (DoWs(t) + ", " + TimeToString(t))          // Format time like "Wed, 2023.02.14 01:59"
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void show()
  {
   datetime tc  = TimeCurrent();
   datetime tts = TimeTradeServer();
   datetime gmt = TimeGMT();

   int BrokerOffset = (int)(tts - gmt);

   // Only show times when the offset change is detected
   if (previous_brokeroffset != -1 && BrokerOffset != previous_brokeroffset) {

      Print("==== Time Change Detected ====");

      PrintFormat("Before - TimeCurrent() = %s  | TimeTradeServer() = %s  | TimeGMT() = %s | BrokerOffset = %d (GMT%+g)",
                  t2s(previous_tc),
                  t2s(previous_tts),
                  t2s(previous_gmt),
                  previous_brokeroffset, previous_brokeroffset/3600.0);

      PrintFormat("After  - TimeCurrent() = %s  | TimeTradeServer() = %s  | TimeGMT() = %s | BrokerOffset = %d (GMT%+g)",
                  t2s(tc),
                  t2s(tts),
                  t2s(gmt),
                  BrokerOffset, BrokerOffset/3600.0);

   }

   // Save previous data
   previous_tc = tc;
   previous_tts = tts;
   previous_gmt = gmt;
   previous_brokeroffset = BrokerOffset;

  }
//+------------------------------------------------------------------+